/* 
 * InCoP - Invisible Camouflage Protocol engine.
 *                                                              
 * Copyright (C) 2011  Luis Campo Giralte 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 2011 
 *
 */

#ifndef _INCOPDBUS_H_
#define _INCOPDBUS_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus.h>
#include <sys/poll.h>
#include "callbacks.h"
#include <sys/types.h>
#include "interfaces.h"

#define INCOPLOG_CATEGORY_NAME INCOP_BUS
#include "log.h"

#define MAX_WATCHES 4

struct ST_InCoPDbusInterface{
	ST_Interface *iface;
	GHashTable *methods;
	GHashTable *properties;
};
typedef struct ST_InCoPDbusInterface ST_InCoPDbusInterface;

struct ST_InCoPDbus{
	DBusWatch *watches[MAX_WATCHES];
	struct pollfd pollfds[MAX_WATCHES];
	int total_watches;
	GHashTable *interfaces;
	GHashTable *private_callbacks;
	GHashTable *properties;
};
typedef struct ST_InCoPDbus ST_InCoPDbus;

void ICDS_Init(void);
void ICDS_Destroy(void);

void ICDS_AddPublicMethod(ST_Interface *iface,ST_Callback *call);
void ICDS_AddPublicProperty(ST_Interface *iface,ST_Callback *call);

void ICDS_AddPrivateCallback(ST_Callback *call);
DBusConnection *ICDS_Connect(char *interface,void *engine);
int ICDS_GetTotalActiveDescriptors(void);
int ICDS_GetDescriptorByIndex(int i);
DBusWatch *ICDS_GetWatchByIndex(int i);
int ICDS_GetEventsByIndex(int i);
void ICDS_Handler(DBusConnection *conn,short events, DBusWatch *watch);
void ICDS_SendSuspiciousSegment(DBusConnection *conn,char *objectname,char *interfacename,char *name,unsigned char *ptr,int length,
	int *start_off, int *end_off,unsigned long hash, u_int32_t seq);
void ICDS_SendVerifiedSegment(DBusConnection *conn,char *objectname,char *interfacename, char *name,
	unsigned long hash, u_int32_t seq,int veredict);

#endif
